#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import progressbar
from git import RemoteProgress


class CloneProgress(RemoteProgress):
    bar = None
    max = 0
    format_custom_text = progressbar.FormatCustomText('%(message)s', dict(message=''))
    widgets = [
        'Téléchargement du logiciel: ', progressbar.Percentage(),
        ' ', progressbar.Bar(),
        ' ', format_custom_text,
    ]

    def update(self, op_code, cur_count, max_count=None, message=''):
        if max_count is not None and self.bar is None or max_count > self.max:
            self.max = max_count
            self.bar = progressbar.ProgressBar(max_value=max_count, widgets=self.widgets)
        elif self.bar is not None:
            self.bar.update(cur_count)
        self.format_custom_text.update_mapping(message=message)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pathlib import Path

CONFIG_PATH = str(Path.home()) + "/allo-config.dict"
ALLO_URL = "allo.dev.libriciel.fr"
API_PATH = "https://{}/api/client".format(ALLO_URL)



# -*- coding: utf-8 -*-
import pickle

from PyInquirer import prompt
from .model.config import AlloConfig
from .model.version import Version
from .model.colors import BColors
from .const import *
from .alloapi import AlloAPI
from .ansible import AlloAnsible


class ConfigLoader:
    config: AlloConfig

    __error_config_msg = "Erreur de configuration, sortie du programme Allo"

    def __init__(self, env):
        try:
            with open(CONFIG_PATH, 'rb') as config_dictionary_file:
                self.config = pickle.load(config_dictionary_file)
        except FileNotFoundError:
            self.config = AlloConfig()
            self.config.env = env
            self.ask_user_configuration()
            # Create specific user to access node
            AlloAnsible.do_add_product_user(self.config.code_produit)

    def ask_user_configuration(self):
        print(BColors.OKBLUE + "Merci de rentrer les informations suivantes afin d'initialiser Allo :")
        self.ask_for_client_info()
        
        versions = AlloAPI.list_version(self.config)
        if not versions:
            print(BColors.FAIL + self.__error_config_msg)
            exit(1)
        else:
            qversions = []
            for v in versions:
                qversions.append({"value": "0", "name": v['name']})
            questions = [{'type': 'list',
                          'name': 'version',
                          'message': 'Version du produit',
                          'choices': qversions}]
            answers = prompt(questions)
            self.config.version = Version(versions[int(answers['version'])])

        if self.verify():
            self.config.teleport_token = prompt(
                [{'type': 'input', 'name': 'teleport_token', 'message': 'Token de télémaintenance'}])['teleport_token']
            self.config.repo_path = prompt(
                [{'type': 'input', 'name': 'repo_path', 'message': 'Chemin d\'installation du logiciel'}])['repo_path']
            self.save()
        else:
            print(BColors.FAIL + "Erreur de configuration, sortie du programme Allo")
            exit(1)

    def verify(self):
        status = AlloAPI.register(self.config)
        if not status:
            print(BColors.FAIL + self.__error_config_msg)
            exit(1)
        if not status['active']:
            cfg = prompt([{'type': 'input', 'name': 'pin_code', 'message': 'Code PIN d\'activation Allo'}])
            AlloAPI.activate(self.config, cfg['pin_code'])
        print(BColors.OKGREEN + "Connexion Allo OK")
        return True

    def ask_for_client_info(self):
        questions = [{'type': 'input', 'name': 'id_client', 'message': 'Identifiant unique Client'},
                     {'type': 'input', 'name': 'code_produit', 'message': 'Code Produit'}]
        cfg = prompt(questions)
        self.config.id_client = cfg['id_client']
        self.config.code_produit = cfg['code_produit']

    def save(self):
        with open(CONFIG_PATH, 'wb') as config_dictionary_file:
            pickle.dump(self.config, config_dictionary_file)



# -*- coding: utf-8 -*-

from .channel import Channel


class Version:
    id: int
    name: str
    git_path: str
    channel: Channel

    def __init__(self, obj):
        self.id = obj['id']
        self.name = obj['name']
        self.git_path = obj['gitPath']
        self.channel = Channel(obj['channel'])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import requests
import jsons
from requests import Response
from uuid import getnode
from .const import *
from .model.colors import BColors
from .model.config import AlloConfig


class AlloAPI:
    @staticmethod
    def list_version(config: AlloConfig):
        r = requests.get("{}/{}/versions".format(API_PATH, config.code_produit),
                         headers={'channel': config.env})
        if AlloAPI.has_error(r):
            return False
        versions = r.json()
        if 'status' in versions:
            print(BColors.FAIL + "{} : {}".format(config.code_produit, versions['message']))
            return False
        elif len(versions) is 0:
            print(BColors.FAIL + "{} : {}".format(config.code_produit, "Aucune versions disponibles"))
            return False
        return versions

    @staticmethod
    def register(config: AlloConfig):
        r = requests.post("{}/{}/register".format(API_PATH, getnode()), json={
            "idClient": config.id_client,
            "product": config.code_produit,
            "version": jsons.dump(config.version)
        })
        if AlloAPI.has_error(r):
            return False
        data = r.json()
        if 'status' in data:
            print(BColors.FAIL + "{} pour l'identifiant {} et le produit {}".format(data['message'],
                                                                                    config.id_client,
                                                                                    config.code_produit))
            return False
        # Update id_client in case we sent something incomplete to the server
        config.id_client = data['idClient']
        config.id = data['id']
        return data

    @staticmethod
    def activate(config: AlloConfig, pin_code: str):
        r = requests.post("{}/{}/activate".format(API_PATH, getnode()), json={
            "idClient": config.id_client,
            "product": config.code_produit,
            "version": jsons.dump(config.version),
            'pin': pin_code
        })
        if AlloAPI.has_error(r):
            return False
        data = r.json()
        if not data['active']:
            print(BColors.FAIL + "Mauvais code PIN")
            return False
        print(BColors.OKGREEN + "Code PIN OK")
        return True

    @staticmethod
    def get_git_token(config: AlloConfig):
        # Get deploy token
        r = requests.get(
            "{}/{}/{}/{}/{}/{}/token".format(API_PATH,
                                             config.id_client,
                                             getnode(),
                                             config.code_produit,
                                             config.version.name,
                                             config.version.channel
                                             ))
        if AlloAPI.has_error(r):
            return False
        data = r.json()
        return data['token']

    @staticmethod
    def has_error(r: Response):
        if r.status_code is 502:
            print(BColors.FAIL + "Erreur lors de la communication avec Allo Server : Erreur {}".format(r.status_code))
            return True
        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from git import Repo, NoSuchPathError, InvalidGitRepositoryError
import logging as log
from .cloneprogress import CloneProgress
from ..model.config import AlloConfig
from ..model.colors import BColors
from ..upgrader import AlloUpgrader
from ..const import *

log.basicConfig(level=log.INFO)


class AlloGit:
    _repo: Repo = None

    token = None
    config: AlloConfig
    upgrader: AlloUpgrader

    def __init__(self, config: AlloConfig, token):
        self.config = config
        self.token = token
        self.init_repo()

    def init_repo(self):
        try:
            self._repo = Repo(self.config.repo_path)
            self._repo.remote().set_url("https://{}:{}@{}/gitlab/{}.git".format(
                    self.config.id_client, self.token, ALLO_URL, self.config.version.git_path))
            self.upgrader = AlloUpgrader(self.config.repo_path)
        except (NoSuchPathError,  InvalidGitRepositoryError):
            os.makedirs(self.config.repo_path, exist_ok=True)
            self._repo = Repo.clone_from(
                "https://{}:{}@{}/gitlab/{}.git".format(
                    self.config.id_client, self.token, ALLO_URL, self.config.version.git_path),
                self.config.repo_path, progress=CloneProgress(), multi_options=['--depth 1'])
            log.info("We have to send event for 'installation'")
            self.upgrader = AlloUpgrader(self.config.repo_path)
            if self.upgrader.do_install() == 0:
                log.info("Version {} installée".format(self._repo.git.describe(tags=True)))
            else:
                print("Erreur d'installation")

        print(BColors.OKGREEN + "✔ {} Version {} detectée".format(self.config.code_produit, self.get_current_version()))

    def get_current_version(self):
        return self._repo.git.describe(tags=True)

    def list_versions(self):
        # Fetch new tags
        self._repo.git.fetch(tags=True)
        return self._repo.tags

    def get_versions_to_pass(self, version: str, reverse: bool = False):
        versions_list = reversed(self.list_versions()) if reverse else self.list_versions()
        versions_to_pass = []
        has_passed_current = False
        for ver in versions_list:
            if has_passed_current:
                versions_to_pass.append(str(ver))
            if str(ver) == self.get_current_version():
                has_passed_current = True
            if version == str(ver):
                break
        return versions_to_pass

    def downgrade_to(self, version: str):
        versions_to_pass = self.get_versions_to_pass(version, True)
        for version in versions_to_pass:
            self.upgrade_to_specific(version)

    def upgrade_to(self, version: str):
        versions_to_pass = self.get_versions_to_pass(version)

        for version in versions_to_pass:
            before_update_version = self.get_current_version()
            if self.upgrade_to_specific(version) != 0:
                log.error("Rollback vers la version {}".format(before_update_version))
                self.upgrade_to_specific(before_update_version)

        log.info("Upgrading to version {} complete with success".format(version))

    def upgrade_to_specific(self, version: str):
        self._repo.git.checkout("tags/{}".format(version))
        return self.upgrader.do_upgrade(version)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from PyInquirer import prompt
from .model.config import AlloConfig
from .model.colors import BColors
from .configloader import ConfigLoader
from .telem import AlloTelem
from .git.gitutils import AlloGit
from .alloapi import AlloAPI


class TestingAllo:
    config: AlloConfig
    telem: AlloTelem
    git: AlloGit

    def __init__(self, env):
        self.config = ConfigLoader(env).config
        self.telem = AlloTelem(self.config)
        # self.git = AlloGit(self.config, AlloAPI.get_git_token(self.config))

        self.what_to_do()

    def what_to_do(self):
        questions = [{'type': 'list', 'name': 'action', 'message': 'Que voulez-vous faire ?',
                      'choices': [
                          'Verifier la connexion',
                          'Ouvrir la télémaintenance',
                          # 'Modifier les informations de connexion',
                          # 'Mettre à jour',
                          # 'Annuler une mise à jour'
                      ]}]
        answers = prompt(questions)
        if 'action' not in answers:
            return
        if 'Verifier la connexion' in answers['action']:
            AlloAPI.register(self.config)
        if 'Ouvrir la télémaintenance' in answers['action']:
            self.telem.connect()
        if 'Modifier les informations de connexion' in answers['action']:
            print("TODO")
        if 'Mettre à jour' in answers['action']:
            self.change_version(True)
        if 'Annuler une mise à jour' in answers['action']:
            self.change_version(False)
        self.what_to_do()

    def change_version(self, upgrade: bool):
        choices = self.git.get_versions_to_pass("", not upgrade)
        if len(choices) > 0:
            questions = [{'type': 'list',
                          'name': 'version',
                          'message': 'Mettre à jour vers quelle version ?' if upgrade else 'Revenir à quelle version ?',
                          'choices': choices}]
            ans = prompt(questions)
            self.git.upgrade_to(ans['version']) if upgrade else self.git.downgrade_to(ans['version'])
        else:
            print(BColors.OKBLUE +
                  ("=> Vous êtes déjà en dernière version" if upgrade else "=> Aucune version sur laquelle revenir"))


def launch():
    import sys
    env = "PROD"
    if len(sys.argv) > 1:
        env = sys.argv[1]
    TestingAllo(env)



# -*- coding: utf-8 -*-

from .version import Version


class AlloConfig:
    id: int
    id_client: str
    code_produit: str
    teleport_token: str
    env: str
    repo_path: str
    version: Version



import os


class AlloAnsible:
    @staticmethod
    def do_install_dependencies():
        corepath = os.path.dirname(os.path.realpath(__file__))
        command = "ansible-playbook {}/playbooks/install_allo.yml".format(corepath)
        oscall_result = os.system(command)
        if os.WEXITSTATUS(oscall_result) != 0:
            print("Error on dependencies installation")
            return os.WEXITSTATUS(oscall_result)
        return 0

    @staticmethod
    def do_add_product_user(product):
        corepath = os.path.dirname(os.path.realpath(__file__))
        command = "ansible-playbook " \
                  "--extra-vars \"user_name=libriciel-{}\" " \
                  "{}/playbooks/create_user.yml".format(product.lower(), corepath)
        oscall_result = os.system(command)
        if os.WEXITSTATUS(oscall_result) != 0:
            print("Error on user creation")
            return os.WEXITSTATUS(oscall_result)
        return 0



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os


def do_install_dependencies():
    corepath = os.path.dirname(os.path.realpath(__file__))
    command = "ansible-playbook " + corepath + "/playbooks/install_allo.yml"
    oscall_result = os.system(command)
    if os.WEXITSTATUS(oscall_result) != 0:
        print("Error on dependencies installation")
        return os.WEXITSTATUS(oscall_result)
    return 0
